# 存储过程

- 一组预先编译好的 SQL 语句的集合，理解成批处理语句

## 创建存储过程

```mysql
CREATE PROCEDURE 存储过程名(参数列表)
BEGIN
	存储过程体（一组合法的SQL语句）
END
```

### 参数列表

- 参数列表包含三部分：参数模式，参数名，参数类型

- in：该参数可以作为输入，也就是该参数需要调用方传入值
- out：该参数可以作为输出，也就是该参数可以作为返回值
- inout：该参数既可以作为输入又可以作为输出，也就是该参数既需要传入值，又可以返回值

### DELIMITER

- DELIMITER 用来设置 MYSQL 的结束符，因为 MYSQL 默认的语句结束符为分号`;` ，存储过程中的SQL语句需要分号来结束，为了避免与存储过程中 SQL 语句结束符相冲突，需要使用 DELIMITER 改变存储过程的结束符，并以"END 结束符"结束存储过程。存储过程定义完毕之后再使用 `DELIMITER ;` 恢复默认结束符。DELIMITER 可以指定任意符号为结束符 
- DELIMITER 是分割符的意思，其实就是定义了一个语句执行的结束符，类似函数 o r存储过程这样的 create 语句，由于其中包含了很多的 `;`，而默认MySQL的结束符就是`;`，那么当我们创建的时候就会报错，有了 DELIMITER 就可以告诉 mysql 解释器，该段命令已经结束了，mysql可以执行

> 注意：当使用DELIMITER命令时，应该避免使用反斜杠（\）字符，因为反斜杠是MYSQL的转义字符！！！

### 调用存储过程

```mysql
CALL 存储过程名(实参列表);
```

#### 调用空参列表的存储过程

```mysql
# 插入到admin表中五条记录
DELIMITER $
CREATE PROCEDURE myp()
BEGIN
	INSERT INTO admin(username,`password`) 
	VALUES('john1','0000'),('lily','0000'),('rose','0000'),('jack','0000'),('tom','0000');
END $
DELIMITER ;

# 调用
CALL myp()；
```

#### 调用带 in 模式参数存储过程

```mysql
# 创建存储过程实现，用户是否登录成功
DELIMITER $
CREATE PROCEDURE myp(IN username VARCHAR(20),IN PASSWORD VARCHAR(20))
BEGIN
	DECLARE result INT DEFAULT 0;
	SELECT COUNT(*) INTO result
	FROM admin
	WHERE admin.username = username
	AND admin.password = PASSWORD;
	SELECT IF(result>0,'登陆成功','登录失败');
END $
DELIMITER ;

# 调用
CALL myp('张飞','8888')；
```

#### 创建 out 模式参数的存储过程

```mysql
# 根据输入的女神名，返回对应的男神名和魅力值
DELIMITER $
CREATE PROCEDURE myp(IN beautyName VARCHAR(20),OUT boyName VARCHAR(20),OUT usercp INT) 
BEGIN
	SELECT boys.boyname ,boys.usercp INTO boyname,usercp
	FROM boys 
	RIGHT JOIN
	beauty b ON b.boyfriend_id = boys.id
	WHERE b.name=beautyName ;	
END $
DELIMITER ;

#调用
CALL myp('小昭',@name,@cp);
SELECT @name,@cp;
```

#### 创建带 inout 模式参数的存储过程

```mysql
# 传入a和b两个值，最终a和b都翻倍并返回
DELIMITER $
CREATE PROCEDURE myp(INOUT a INT ,INOUT b INT)
BEGIN
	SET a=a*2;
	SET b=b*2;
END $
DELIMITER ;

# 调用
SET @m=10;
SET @n=20;
CALL myp(@m,@n);
SELECT @m,@n;
```

### 删除存储过程

```mysql
DROP PROCEDURE 存储过程名;
```

### 查看存储过程

```mysql
SHOW CREATE PROCEDURE 存储过程名;
```

