# 自定义函数

- 一组预先编译好的 SQL 语句的集合，理解成批处理语句

## 自定义函数与存储过程区别

- 存储过程：可以有0个返回，也可以有多个返回，适合做批量插入、批量更新

- 函数：有且仅有1 个返回，适合做处理数据后返回一个结果

## 创建自定义函数

```mysql
CREATE FUNCTION 函数名(参数列表) RETURNS 返回类型
BEGIN
	函数体
END
```

> 参数列表 包含两部分：参数名，参数类型
>
> 必须要有 return

## 调用自定义函数

```mysql
SELECT 函数名(参数列表)
```

### 调用无参有返回的自定义函数

```mysql
# 案例: 返回公司的员工个数
DELIMITER $
CREATE FUNCTION myf1() RETURNS INT
BEGIN

	DECLARE c INT DEFAULT 0;
	SELECT COUNT(*) INTO c
	FROM employees;
	RETURN c;
	
END $
DELIMITER ;

# 调用
SELECT myf1();
```

### 有参有返回的自定义函数

```mysql
# 案例1: 根据员工名，返回它的工资
DELIMITER $
CREATE FUNCTION myf2(empName VARCHAR(20)) RETURNS DOUBLE
BEGIN
	SET @sal=0;#定义用户变量 
	SELECT salary INTO @sal   #赋值
	FROM employees
	WHERE last_name = empName;
	
	RETURN @sal;
END $
DELIMITER ;

# 调用
SELECT myf2('k_ing'); 


#案例2：根据部门名，返回该部门的平均工资
DELIMITER $
CREATE FUNCTION myf3(deptName VARCHAR(20)) RETURNS DOUBLE
BEGIN
	DECLARE sal DOUBLE ;
	SELECT AVG(salary) INTO sal
	FROM employees e
	JOIN departments d ON e.department_id = d.department_id
	WHERE d.department_name=deptName;
	RETURN sal;
END $
DELIMITER ;

# 调用
SELECT myf3('IT')；
```

### 查看函数

```mysql
SHOW CREATE FUNCTION 函数名;
```

### 删除函数

```mysql
DROP FUNCTION 函数名;
```

