# 流程控制

## 分支结构

### IF 函数

```mysql
if(条件,值1,值2)
```

### CASE 结构

```mysql
CASE 变量或表达式
WHEN 值1 THEN 语句1;
WHEN 值2 THEN 语句2;
...
ELSE 语句n;
END 
```

```mysql
CASE 
WHEN 条件1 THEN 语句1;
WHEN 条件2 THEN 语句2;
...
ELSE 语句n;
END 
```

> 应用在 BEGIN END 中或外面

#### 案例

```mysql
# 创建函数，实现传入成绩，如果成绩>90,返回A，如果成绩>80,返回B，如果成绩>60,返回C，否则返回D
DELIMITER $
CREATE FUNCTION test_case(score FLOAT) RETURNS CHAR
BEGIN 
	DECLARE ch CHAR DEFAULT 'A';
	CASE 
	WHEN score>90 THEN SET ch='A';
	WHEN score>80 THEN SET ch='B';
	WHEN score>60 THEN SET ch='C';
	ELSE SET ch='D';
	END CASE;
	RETURN ch;
END $
DELIMITER ;

SELECT test_case(56);
```

### IF 结构

```mysql
IF 条件1 THEN 语句1;
ELSEIF 条件2 THEN 语句2;
....
ELSE 语句n;
END IF;
```

> 只能应用在 BEGIN END 中

#### 案例

```mysql
# 创建函数，实现传入成绩，如果成绩>90,返回A，如果成绩>80,返回B，如果成绩>60,返回C，否则返回D
DELIMITER $
CREATE FUNCTION test_if(score FLOAT) RETURNS CHAR
BEGIN
	DECLARE ch CHAR DEFAULT 'A';
	IF score>90 THEN SET ch='A';
	ELSEIF score>80 THEN SET ch='B';
	ELSEIF score>60 THEN SET ch='C';
	ELSE SET ch='D';
	END IF;
	RETURN ch;
END $
DELIMITER ;

# 调用
SELECT test_if(87);

# 创建存储过程，如果工资<2000,则删除，如果5000>工资>2000,则涨工资1000，否则涨工资500
DELIMITER $
CREATE PROCEDURE test_if_pro(IN sal DOUBLE)
BEGIN
	IF sal<2000 THEN DELETE FROM employees WHERE employees.salary=sal;
	ELSEIF sal>=2000 AND sal<5000 THEN UPDATE employees SET salary=salary+1000 WHERE employees.`salary`=sal;
	ELSE UPDATE employees SET salary=salary+500 WHERE employees.`salary`=sal;
	END IF;
	
END $
DELIMITER ;

# 调用
CALL test_if_pro(2100)；
```

## 循环结构

### 分类

- while
- loop
- repeat

### 循环控制

- iterate：类似于 continue，继续，结束本次循环，继续下一次
- leave：类似于 break，跳出，结束当前所在的循环

### WHILE

```mysql
WHILE 循环条件 DO
	循环体;
END WHILE;
```

### LOOP

```mysql
LOOP 
循环体;
END LOOP;
```

### REPEAT

```mysql
REPEAT 
循环体;
UNTIL 结束循环的条件
END REPEAT
```

### 案例

#### 简单示例

```mysql
# 案例：批量插入，根据次数插入到admin表中多条记录
DELIMITER $
CREATE PROCEDURE pro_while1(IN insertCount INT)
BEGIN
	DECLARE i INT DEFAULT 1;
	WHILE i<=insertCount DO
		INSERT INTO admin(username,`password`) VALUES(CONCAT('Rose',i),'666');
		SET i=i+1;
	END WHILE;
	
END $
DELIMITER ;

# 调用
CALL pro_while1(100);
```

#### 配合 LEAVE 语句

```mysql
#案例：批量插入，根据次数插入到admin表中多条记录，如果次数>20则停止
DELIMITER $
CREATE PROCEDURE test_while1(IN insertCount INT)
BEGIN
	DECLARE i INT DEFAULT 1;
	a:WHILE i<=insertCount DO
		INSERT INTO admin(username,`password`) VALUES(CONCAT('xiaohua',i),'0000');
		IF i>=20 THEN LEAVE a;
		END IF;
		SET i=i+1;
	END WHILE a;
END $
DELIMITER ;

# 调用
CALL test_while1(100)；
```

#### 配合 iterate 语句

```mysql
#案例：批量插入，根据次数插入到admin表中多条记录，只插入偶数次
DELIMITER $
CREATE PROCEDURE test_while1(IN insertCount INT)
BEGIN
	DECLARE i INT DEFAULT 0;
	a:WHILE i<=insertCount DO
		SET i=i+1;
		IF MOD(i,2)!=0 THEN ITERATE a;
		END IF;
		INSERT INTO admin(username,`password`) VALUES(CONCAT('xiaohua',i),'0000');
	END WHILE a;
END $
DELIMITER ;

# 调用
CALL test_while1(100);
```

