# 查询截取分析

## 慢查询日志

- MySQL 的慢查询日志是 MySQL提供的一种日志记录，它用来记录在 MySQL 中响应时间超过阀值的语句，具体指运行时间超过 long_query_time 值的SQL，则会被记录到慢查询日志中
- long_query_time 的默认值为 10，意思是运行10秒以上的语句
- 查看哪些 SQL 超出了我们的最大忍耐时间值，比如一条sql执行超过5秒钟，我们就算慢 SQL，希望能收集超过5秒的sql，结合之前explain进行全面分析
- 默认情况下，MySQL 数据库没有开启慢查询日志，需要我们手动来设置这个参数
- 当然，如果不是调优需要的话，一般不建议启动该参数，因为开启慢查询日志会或多或少带来一定的性能影响

**查看慢日志是否开启**

```mysql
SHOW VARIABLES LIKE '%slow_query_log%';
```

**开启慢日志**

```mysql
set global slow_query_log=1;
```

**永久生效**

```shell
#[mysqld]下配置：
slow_query_log=1;
slow_query_log_file=/var/lib/mysql/atguigu-slow.log
long_query_time=3;
log_output=FILE
```

## mysqldumpslow

```mysql
# 得到返回记录集最多的10个SQL
mysqldumpslow -s r -t 10 /var/lib/mysql/my-slow.log
 
# 得到访问次数最多的10个SQL
mysqldumpslow -s c -t 10 /var/lib/mysql/my-slow.log
 
# 得到按照时间排序的前10条里面含有左连接的查询语句
mysqldumpslow -s t -t 10 -g "left join" /var/lib/mysql/my-slow.log
 
# 另外建议在使用这些命令时结合 | 和more 使用 ，否则有可能出现爆屏情况
mysqldumpslow -s r -t 10 /var/lib/mysql/my-slow.log | more
```

## Show Profile

是 MySQL 提供可以用来分析当前会话中语句执行的资源消耗情况。可以用于 SQL 的调优的测量

**查看功能是否开启**

```mysql
show variables like 'profiling';
```

**开启 profiling**

```mysql
SET profiling=1;
```

**查看结果**

```mysql
show profiles
```

**诊断 SQL**

```mysql
show profile cpu,block io for query 上一步结果的Query_Id
```

**分析结果**

- converting HEAP to MyISAM：查询结果太大，内存都不够用，开始占用磁盘
- Creating tmp table：创建了临时表
- Copying to tmp table on disk：把内存中临时表复制到磁盘，极其损耗性能
- locked：被锁

## 全局查询日志

**配置启用**

```mysql
# 在mysql的my.cnf中，设置如下
general_log=1   
# 记录日志文件的路径
general_log_file=/path/logfile
# 输出格式
log_output=FILE
```

**编码启用**

```mysql
set global general_log=1;

# 设置查询日志存储到表中
set global log_output='TABLE';
# 在数据库查询全局日志
select * from mysql.general_log;
```

> 尽量不要在生产环境开启这个功能