# 复杂度

- 一般用大 O 表示法来描述复杂度，它表示的是数据规模 n 对应的复杂度
- 忽略常数，系数，低阶
  - 9 >> O(1)
  - 2n + 3 >> O(n)
  - n^2 + 2n +6 >> O(n^2)
- 对数阶一般省略底数，统称为 log(n)

> 注意：大 O 表示法仅仅是一种粗略的分析模型，是一种估算，能够帮助我们快速了解一个算法的执行效率

## 常见的复杂度

| 执行次数               | 复杂度   | 非正式术语 |
| ---------------------- | -------- | ---------- |
| 12                     | O(1)     | 常数阶     |
| 2n + 3                 | O(n)     | 线性阶     |
| 4n^2 + 2n + 6          | O(n^2)   | 平方阶     |
| 4log2n + 25            | O(logn)  | 对数阶     |
| 3n + 2nlog3n + 15      | O(nlogn) | nlogn阶    |
| 4n^3 + 3n^2 + 22n +100 | O(n^3)   | 立方阶     |
| 2^n                    | O(2^n)   | 指数阶     |

## 斐波那契数

- 斐波那契数列指的是这样一个数列 0, 1, 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144, 233，377，610，987，1597，2584，4181，6765，10946，17711，28657，46368……
- 特别指出：第0项是0，第1项是第一个1
- 这个数列从第三项开始，每一项都等于前两项之和

```java
// 复杂度：O(2^n)
public static int fib1(int n) {
    if (n <= 1) return n;
    return fib1(n - 1) + fib1(n - 2);
}
```

```java
// 复杂度：O(n)
public static int fib2(int n) {
   if (n <= 1) return n;
   
   int first = 0;
   int second = 1;
   for (int i = 0; i < n - 1; i++) {
      int sum = first + second;
      first = second;
      second = sum;
   }
   return second;
}
```

> 也可以使用特征函数实现