# tomcat类加载机制

Tomcat的类加载机制是违反了双亲委托原则的，对于一些未加载的非基础类，各个web应用自己的类加载器(WebAppClassLoader)会优先查看自己的仓库加载，加载不到时再交给commonClassLoader走双亲委托

![tomcat-class](https://blog.dreamly.fun/upload/2021/12/tomcat-class-8c965e48fa8449bbad3155aeb3edeb56.png)

**当Tomcat启动时，会创建几种类加载器**

- Bootstrap 引导类加载器：加载JVM启动所需的类，以及标准扩展类（位于jre/lib/ex下）
- System 系统类加载器：加载tomcat启动的类，比如bootstrap.jar，通常在catalinalbat或者catalina.sh中指定，位于`CATALINA_ HOME/bin`下
- commonLoader： Tomcat最基本的类加载器，加载路径中的class可以被Tomcat容器本身以及各个Webapp访问，加载CATALINA_ HOME/ib下的结构，比如servlet-api.jar)
- catalinaloader：Tomcat容器私有的类加载器，加载路径中的class对于Webapp不可见
- sharedLoader： 各个Webapp共享的类加载器，加载路径中的class对于所有Webapp可见，但是对于Tomcat容器不可见
- WebappClassLoader：单个Tomcat实例中各个Web应用程序私有的类加载器，加载路径中的class只对当前Webapp可见（加载WEB-INF/ib和WEB-INF/classes 下的结构）

> CommonClassLoader、CatalinaClassLoader、SharedClassLoader、WebappClassLoader，这些是Tomcat自己定义的类加载器，它们分别加载`/common/`、`/server/`、`/shared/`（在tomcat 6之后己经合并到根目录下的 lib目录下）和`/WebApp/WEB-INF/*`中的Java类库，其中 WebApp 类加载器和 Jsp 类加载器通常会存在多个实例，每一个Web应用程序对应一个WebApp类加载器，一个 JSP 文件对应一个Jsp类加载器

**从图中的委派关系中可以看出**

CommonClassloader 能加载的类都可以被 Catalina ClassLoader 和 SharedClassLoader 使用，从而实现了公有类库的共用，而CatalinaClassLoader和Shared Classloader自己能加载的类则与对方相互隔离

WebAppClassloader可以使用SharedClassLoader加载到的类，但各个WebAppClassLoader实例之间相互隔离

而 JasperLoader 的加载范围仅仅是这个JSP文件所编译出来的那一个class文件，它出现的目的就是为了被丢弃，当Web容器
检测到 JSP 文件被修改时，会替换掉目前的 Jasperloader 的实例，并通过再建立一个新的Jsp类加载器来实现JSP文件的Hotswap功能

**当应用需要到某个类时，则会按照下面的颜序进行类加载**

1. 使用 bootstrap 引导类加载器加载
2. 使用 system 系统类加载器加载
3. 使用应用类加载器在 WEB-INF/classes 中加载
4. 使用应用类加载器在 WEB-INF/lib 中加载
5. 使用common类加载器在CATALINA HOME/lib中加载

