# 1.谈谈Spring IOC的理解，原理与实现?

- 总

  - 控制反转：理论思想，原来的对象是由使用者来进行控制，有了 Spring 之后，可以把整个对象交给 Spring 来帮我们进行管理
  - DI：依赖注入，把对应的属性的值注入到具体的对象中，@Autowired，populateBean 完成属性值的注入
  - 容器：存储对象，使用map结构来存储，在 Spring 中一般存在三级缓存，singletonObjects 存放完整的 bean 对象，整个 bean的生命周期，从创建到使用到销毁的过程全部都是由容器来管理
- 分

  1. 一般聊 IOC 容器的时候要涉及到容器的创建过程（beanFactory,DefaultListableBeanFactory），向 bean 工厂中设置一些参数（BeanPostProcessor，Aware接口的子类）
  2. 加载解析 bean 对象，准备要创建的bean对象的定义对象 beanDefinition，(xml或者注解的解析过程)
  3. beanFactoryPostProcessor 的处理，此处是扩展点，PlaceHolderConfigurSupport、ConfigurationClassPostProcessor
  4. BeanPostProcessor 的注册功能，方便后续对 bean 对象完成具体的扩展功能
  5. 通过反射的方式讲 BeanDefinition 对象实例化成具体的bean对象
  6. bean 对象的初始化过程（填充属性，调用 aware 子类的方法，调用 BeanPostProcessor 前置处理方法，调用 init-mehtod 方法，调用 BeanPostProcessor 的后置处理方法）
  7. 生成完整的 bean 对象，通过 getBean 方法可以直接获取
  8. 销毁过程

# 2.谈一下spring IOC的底层实现

- 先通过 createBeanFactory 创建出一个 Bean 工厂（DefaultListableBeanFactory）

- 开始循环创建对象，因为容器中的 bean 默认都是单例的，所以优先通过 getBean、doGetBean 从容器中查找，找不到的话，通过createBean, doCreateBean 方法，以反射的方式创建对象，一般情况下使用的是无参的构造方法（getDeclaredConstructornewInstance）

- 进行对象的属性填充 populateBean

- 进行其他的初始化操作（initializingBean）


# 3.描述一下bean的生命周期 ？

1. 实例化 bean：反射的方式生成对象
2. 填充bean的属性：populateBean()，循环依赖的问题（三级缓存）
3. 调用 aware 接口相关的方法：invokeAwareMethod (完成BeanName、BeanFactory、eanClassLoader 对象的属性设置)
4. 调用 BeanPostProcessor 中的前置处理方法：使用比较多的有（ApplicationContextPostProcessor，设置ApplicationContext、Environment、ResourceLoader、EmbeddValueResolver等对象）
5. 调用initmethod方法：invokeInitmethod()，判断是否实现了 initializingBean 接口，如果有，调用 afterPropertiesSet 方法
6. 调用 BeanPostProcessor 的后置处理方法：Spring的 AOP 就是在此处实现的：AbstractAutoProxyCreator
7. 注册 Destuction 相关的回调接口

8. 获取到完整的对象，可以通过 getBean 的方式来进行对象的获取
9. 销毁流程：判断是否实现了DispoableBean接口，调用 destroyMethod 方法


# 4.Spring 是如何解决循环依赖的问题的？

- 三级缓存，提前暴露对象，AOP


**形成闭环的原因**

- 先创建 A 对象，实例化 A 对象，此时 A 对象中的 b 属性为空，填充属性 b
- 从容器中查找 B 对象，如果找到了，直接赋值不存在循环依赖问题，找不到直接创建 B 对象
- 实例化 B 对象，此时 B 对象中的 a 属性为空，填充属性 a
- 从容器中查找 A 对象，找不到，直接创建

- 此时，如果仔细琢磨的话，会发现 A 对象是存在的，只不过此时的A对象不是一个完整的状态，只完成了实例化但是未完成初始化，如果在程序调用过程中，拥有了某个对象的引用，能否在后期给他完成赋值操作，可以优先把非完整状态的对象优先暴露，等待后续操作来完成赋值，相当于提前暴露了某个不完整对象的引用，所以解决问题的核心在于实例化和初始化分开操作，这也是解决循环依赖问题的关键

- 当所有的对象都完成实例化和初始化操作之后，还要把完整对象放到容器中，此时在容器中存在对象的 2 个状态，完成实例化但未完成初始化和完整状态，因为都在容器中，所以要使用不同的 Map 结构来进行存储，此时就有了一级缓存和二级缓存，如果一级缓存中有了，那么二级缓存中就不会存在同名的对象，因为他们的查找顺序是 1，2，3 这样的方式来查找的
- 一级缓存中放的是完整对象，二级缓存中放的是非完整对象
- 三级缓存的 value 类型是 ObjectFactory，是一个函数式接口，存在的意义是保证在整个容器的运行过程中同名的 bean 对象唯一


**如果一个对象需要被代理，或者说需要生成代理对象，那么要不要优先生成一个普通对象？**

- 需要
- 普通对象和代理对象是不能同时出现在容器中的，因此当一个对象需要被代理的时候，就要使用代理对象覆盖掉之前的普通对象，在实际的调用过程中，是没有办法确定什么时候对象被使用，所以就要求当某个对象被调用的时候，优先判断此对象是否需要被代理，类似于一种回调机制的实现，因此传入 lambda 表达式的时候，可以通过 lambda 表达式来执行对象的覆盖过程，getEarlyBeanReference()

- 所有的bean对象在创建的时候都要优先放到三级缓存中，在后续的使用过程中，如果需要被代理则返回代理对象，如果不需要被代理，则直接返回普通对象


# 4.1缓存的放置时间和删除时间

​		三级缓存：createBeanInstance之后：addSingletonFactory

​		二级缓存：第一次从三级缓存确定对象是代理对象还是普通对象的时候，同时删除三级缓存 getSingleton

​		一级缓存：生成完整对象之后放到一级缓存，删除二三级缓存:addSingleton

# 5.Bean Factory与FactoryBean有什么区别？

- 相同点：都是用来创建 bean 对象的

- 不同点：使用 BeanFactory 创建对象的时候，必须要遵循严格的生命周期流程，太复杂了，如果想要简单的自定义某个对象的创建，同时创建完成的对象想交给spring来管理，那么就需要实现 FactroyBean 接口了

  - isSingleton：是否是单例对象
  - getObjectType：获取返回对象的类型
  - getObject：自定义创建对象的过程 (new，反射，动态代理)

# 6.Spring中用到的设计模式? 

- 单例模式：bean 默认都是单例的

- 原型模式：指定作用域为 prototype

- 工厂模式：BeanFactory

- 模板方法：postProcessBeanFactory、onRefresh、initPropertyValue

- 策略模式：XmlBeanDefinitionReader、PropertiesBeanDefinitionReader

- 观察者模式：listener、event、multicast

- 适配器模式：Adapter

- 装饰者模式：BeanWrapper

- 责任链模式：使用 AOP 的时候会先生成一个拦截器链

- 代理模式：动态代理

- 委托者模式：delegate


# 7.Spring的AOP的底层实现原理? 

- AOP 是 IOC 的一个扩展功能，先有的 IOC，再有的 AOP，只是在 IOC 的整个流程中新增的一个扩展点而已：BeanPostProcessor
- bean 的创建过程中有一个步骤可以对 bean 进行扩展实现，AOP 本身就是一个扩展功能，所以在 BeanPostProcessor 的后置处理方法中来进行实现

  1. 代理对象的创建过程（advice，切面，切点）
  2. 通过 jdk 或者 cglib 的方式来生成代理对象
  3. 在执行方法调用的时候，会调用到生成的字节码文件中，直接回找到 DynamicAdvisoredInterceptor 类中的 intercept 方法，从此方法开始执行
  4. 根据之前定义好的通知来生成拦截器链
  5. 从拦截器链中依次获取每一个通知开始进行执行，在执行过程中，为了方便找到下一个通知是哪个，会有一个 CglibMethodInvocation 的对象

# 8.Spring的事务是如何回滚的?

Spring 的事务是由 AOP 来实现的，首先要生成具体的代理对象，然后按照 AOP 的整套流程来执行具体的操作逻辑，正常情况下要通过通知来完成核心功能，但是事务不是通过通知来实现的，而是通过一个 TransactionInterceptor 来实现的，然后调用 invoke 来实现具体的逻辑

1. 先做准备工作，解析各个方法上事务相关的属性，根据具体的属性来判断是否开启新事务

2. 当需要开启事务的时候，获取数据库连接，关闭自动提交功能，开启事务

3. 执行具体的 SQL 逻辑操作

4. 在操作过程中，如果执行失败了，那么会通过 completeTransactionAfterThrowing 来完成事务的回滚操作，回滚的具体逻辑是通过doRollBack 方法来实现的，实现的时候也是要先获取连接对象，通过连接对象来回滚

5. 如果执行过程中，没有任何意外情况的发生，那么通过 commitTransactionAfterReturning 来完成事务的提交操作，提交的具体逻辑是通过 doCommit 方法来实现的，实现的时候也是要获取连接，通过连接对象来提交

6. 当事务执行完毕之后需要清除相关的事务信息 cleanupTransactionInfo

7. 如果想要聊的更加细致的话，需要知道 TransactionInfo、TransactionStatus


# 9.谈一下spring事务传播？

​	
