###     centos7安装docker

移除旧的版本：

```
sudo yum remove docker 
                  docker-client 
                  docker-client-latest 
                  docker-common 
                  docker-latest 
                  docker-latest-logrotate 
                  docker-logrotate 
                  docker-selinux 
                  docker-engine-selinux 
                  docker-engine
```

安装一些必要的系统工具：

```shell
sudo yum install -y yum-utils device-mapper-persistent-data lvm2
```

添加软件源信息：

```shell
sudo yum-config-manager --add-repo http://mirrors.aliyun.com/docker-ce/linux/centos/docker-ce.repo
```

更新 yum 缓存：

```shell
sudo yum makecache fast
```

安装 Docker-ce：

```shell
sudo yum -y install docker-ce
```

启动 Docker 后台服务

```shell
sudo systemctl start docker
```

###  镜像操作

#### docker image

```shell
docker images -a：列出本地所有镜像
docker images -q：只显示镜像ID
docker images --digests：显示镜像的摘要信息
docker images --no-trunc：显示完整的镜像信息
```

#### docker search（其实是在https://hub.docker.com/查找镜像）

```shell
docker search + 镜像名字
docker search tomcat --no-trunc：显示完整的镜像描述
docker search -s 30 tomcat：列出收藏数不小于指定值的镜像
```

#### docker pull + 镜像名字（默认不加版本号就是最新版本）

```shell
docker pull tomcat:latest：拉取最新版本的tomcat镜像
```

#### docker rmi

```shell
docker rmi -f hello-world:latest ：强制删除镜像
docker rmi -f hello-world:latest nginx:latest ：删除多个，中间以空格分隔
docker rmi -f $(docker images -qa) ：删除全部
```

### 容器操作

#### docker run

```shell
docker run [OPTIONS] IMAGE [COMMAND] [ARG...] ：新建并启动容器
OPTIONS说明（常用）：有些是-，有些是--
--nanme=[为镜像指定名字]
-d：代表后台运行，并返回容器ID,也即启动守护式容器
-i:以交互模式运行容器，通常与-t同时使用
-t：为容器重新分配一个伪终端
-P：随机端口映射
-p：指定端口映射，有4种格式
ip:hostPort:containerPort
ip::containerPort
hostPort:containerPort
containerPort
举例：
docker run -it --name mycentos01 centos：（交互式启动）
docker run -it mycentos:1.3 ：（加版本号，默认不加就是latest，例如上面的）
```

#### docker ps [OPTIONS]（不加参数默认显示当前运行的容器）

```shell
OPTIONS参数说明:
-a：列出当前所有正在运行的容器 + 历史上运行过的
-l：显示最近创建的容器
-n：显示最近n个创建的容器
-q：静默模式，只显示容器编号
--no-trunc：不截断输出
```

#### 退出容器

```shell
exit：容器停止退出
ctrl+P+Q：容器不停止退出
```

#### 启动容器

```shell
docker start + 容器名/容器ID
docker restart + 容器名/容器ID ：表示重启
```

#### 停止容器

```shell
docker stop 容器名/容器ID
docker kill 容器名/容器ID ：强制停止
```

#### 删除容器

```shell
docker rm 容器名/容器ID
docker rm -f 容器名/容器ID ：强制删除
docker rm -f$(docker ps -a -q) ：强行删除多个容器
```

#### 守护式启动容器

```shell
docker run -d centos
问题：然后docker ps -a 进行查看，会发现容器已经退出
很重要的要说明一点：docker容器后台运行，就必须有一个前台进程。
容器运行的命令如果不是那些一直挂起的命令（比如运行top，tail），会自动退出，这是docker的机制
解决：
docker run -d centos /bin/sh -c "while true;do echo hello world;sleep 2;done"
```

#### 查看容器日志

```shell
docker logs -f -t --tail 容器ID/容器名
-t ：加入时间戳
-f ：跟随最新的日志打印
--tail ：数字，显示最后多少条
```

#### 查看容器内进程

```shell
docker top 容器ID/容器名
```

#### 查看容器内部细节

```shell
docker inspect 容器ID/容器名
```

#### 进入正在运行的容器并以命令行交互

```shell
docker exec -it 容器ID/容器名 bashShell命令
docker attach 容器ID/容器名
attach ：直接进入到容器启动命令的终端，不会启动新的进程
exec：在容器中打开新的终端，并且可以启动新的进程
docker exec -it centos ls -l/tmp ：在宿主机显示结果
docker exec -it centos /bin/bash ：进入到容器终端（跟attach一个效果）
```

#### 从容器中拷贝文件到宿主机

```shell
docker cp 容器ID:容器内路径 目的主机路径
```

### docker镜像

**镜像：**镜像是一种轻量级，可执行的独立软件包，用来打包软件运行环境和基于运行环境开发的软件，它包含运行某个软件所需的所有内容，包括代码，运行时库，环境变量和配置文件

**UnionFs**（联合文件系统）：联合文件系统（[UnionFS](http://en.wikipedia.org/wiki/UnionFS)）是一种分层、轻量级并且高性能的文件系统，它支持对文件系统的修改作为一次提交来一层层的叠加，同时可以将不同目录挂载到同一个虚拟文件系统下(unite several directories into a single virtual filesystem)。联合文件系统是 Docker 镜像的基础。镜像可以通过分层来进行继承，基于基础镜像（没有父镜像），可以制作各种具体的应用镜像。

**特性**：一次同时加载多个文件系统，但从外面看，只能看到一个文件系统，联合加载就会把各层文件系统叠加起来，这样最终的文件系统，会包含所有的底层文件和目录。

**加载原理：**典型的Linux启动到运行需要两个FS ，bootfs + rootfs (从功能角度而非文件系统角度)。

bootfs (boot file system) 主要包含 bootloader 和 kernel, bootloader主要是引导加载kernel, 当boot成功后 kernel 被加载到内存中后 bootfs就被umount了.

rootfs (root file system) 包含的就是典型 Linux 系统中的 /dev, /proc, /bin, /etc 等标准目录和文件。实际上容器也就是跑了rootfs，所以比传统虚拟机快的多

**分层镜像**：共享资源，多个镜像都是从相同的base镜像构建而来，那么宿主机只需在磁盘上保存一份base镜像，同时内存中也只需加载一份base镜像，就可以为所有容器服务了，而且镜像的每一层都可以被共享

#### docker 启动tomcat

```shell
docker run -it -p:8080:8080 tomcat ：（前台方式）
docker run -d -p 1111:8080 tomcat ：（后台方式）
-p 主机端口:docker容器端口
i：交互
t：终端
```

**docker commit**（提交容器副本使之成为一个新的镜像）

```shell
docker commit -a="mrzhou" -m="del tomcat docs" 容器ID mrzhou/mytomcat:1.0
-a ：作者
-m ：描述
```

### docker容器数据卷

类似Redis中的rdb和aof文件，卷的设计目的就是数据的持久化，完全独立与容器的生存周期，因此docker不会在容器删除的时删除其挂载的数据卷

#### 数据卷的添加

**命令添加**

```shell
docker run -it -v /宿主机绝对路径目录:/容器内路径 镜像名
#带权限的命令(read only)，默认不加是rw
docker run -it -v /宿主机绝对路径目录:/容器内路径:ro 镜像名
#如果遇到权限报错，可以加这样的参数
docker run -it -v /宿主机绝对路径目录:/容器内路径 --privileged=true 镜像名
```

**DockerFile添加**

1. 创建一个file，将内容写进去

```dockerfile
#volume test
FROM centos
VOLUME ["/dataVOlumeContainer1","/dataVOlumeContainer2"]
CMD echo "finished,--------success1"
CMD /bin/bash
```

2. 执行命令

```shell
#centos指的是镜像的名字或者id
docker build -f /mydocker/Dockerfile -t zzjj/centos .
```

3. 执行完可以使用docker images查看构建的容器
4. 运行容器，在dataVOlumeContainer1中创建文件
5. 使用docker inspect 镜像ID查看（可以看到宿主机默认的映射地址）

```json
            {
                "Type": "volume",
                "Name": "9da5ce74173b604716da101186961671fc40a065abbd6558be46ca5b778c171a",
                "Source": "/www/server/docker/volumes/9da5ce74173b604716da101186961671fc40a065abbd6558be46ca5b778c171a/_data",
                "Destination": "/dataVOlumeContainer1",
                "Driver": "local",
                "Mode": "",
                "RW": true,
                "Propagation": ""
            },
            {
                "Type": "volume",
                "Name": "4a720f577a1301403f0a6dec7e7ee1f296af6a6de91f0933c1442e018b10301e",
                "Source": "/www/server/docker/volumes/4a720f577a1301403f0a6dec7e7ee1f296af6a6de91f0933c1442e018b10301e/_data",
                "Destination": "/dataVOlumeContainer2",
                "Driver": "local",
                "Mode": "",
                "RW": true,
                "Propagation": ""
            }

```

#### 容器数据卷

容器之间数据共享（volumes-from）

```SHELL
docker run -it --name doc01 zzjj/centos
docker run -it --name doc02 --volumes-from doc01 zzjj/centos
```



容器之间配置信息的传递，数据卷的生命周期一直持续到没有容器使用它为止

### DockerFile解析

DockerFile是用来构建Docker镜像的构建文件，是由一系列命令和参数构成的脚本

#### Docker内容基础知识

1. 每条保留字指令都必须为大写字母，而且后面要跟随至少一个参数
2. 指令按照从上到下，顺序执行
3. 【#】代表注释
4. 每条指令都会创建一个新的镜像层，并对镜像进行提交

#### Docker执行DockerFile的大致流程

1. docker从基础镜像运行一个容器
2. 执行一条指令并对容器做出修改
3. 执行类似docker commit的操作提交一个新的镜像层
4. docker再基于刚提交的镜像运行一个新容器
5. 执行dockerfile中的下一条指令直到所有指令完成

**总结**

从应用软件的角度来看，dockerfile，docker镜像与docker容器分别代表软件的3个不同阶段

- dockerfile是软件的原材料
- docker镜像是软件的交付品
- docker容器是软件的可运行状态

dockerfile面向开发，docker镜像成为交付标准，docker容器则涉及到部署和运维，三者缺一不可。

#### docker体系结构（保留字指令）

FROM：基础镜像，当前新镜像是基于哪个镜像

MAINTAINER：镜像维护者的姓名和邮箱地址

RUN：容器构建时需要运行的命令

EXPOSE：当前容器对外暴露的端口

WORKDIR：指定在创建容器后，终端默认登录进来的工作目录，一个落脚点

ENV：用来在构建镜像的过程中设置环境变量

ADD：将宿主机目录下的文件拷贝到镜像而且ADD指令会自动处理URL和解压tar压缩包

COPY：类似ADD，拷贝文件和目录到镜像中，将从构建上下文目录中<源目录>的文件/目录复制到新的一层的镜像内的<目标路径>位置

VOLUME：容器数据卷，用来数据保存和持久化工作

CMD：指定一个容器启动时要运行的命令，dockerfile中可以有多个CMD指令，但是只有最后一个生效，CMD会被docker run之后的参数值替换

ENTRYPOINT：指定一个容器启动时要运行的命令，ENTRYPOINT的目的和CMD一样，都是在指定容器启动程序及参数

ONBUILD：当构建一个被继承的dockerfile时运行命令，父镜像在被子继承后父镜像的ONBUILD被触发

#### 案例

Base镜像（scratch）：docker hub中99%的镜像都是通过在base镜像中安装和配置需要的软件构建出来的

##### 自定义镜像mycentos

```dockerfile
FROM centos
MAINTAINER zzjj<1782920040@qq.com>

ENV MYPATH /usr/local
WORKDIR $MYPATH

RUN yum -y install vim
RUN yum -y install net-tools

EXPOSE 80

CMD echo $MYPATH
CMD echo "SUCCESS-----------OK"
CMD /bin/bash
```

执行build

```shell
docker build -f /mydocker/Dockerfile -t mycentos:1.3 .
```

##### CMD/ENTRYPOINT镜像

它们都是指定一个容器启动时要运行的命令

CMD：dockerfile中可以有多个CMD命令，但是只有最后一个生效，CMD会被docker run之后的参数替换

ENTRYPOINT：docker run之后的参数会被当作参数传递给ENTRYPOINT，之后形成新的命令组合

例子：

```dockerfile
FROM centos
RUN yum install -y curl
ENTRYPOINT ["curl","-s","http://ip.cn"]
```

```shell
docker build -f /mydocker/Dockerfile -t myip .
```

```shell
#这个指令相当于 ENTRYPOINT ["curl","-s","-i","http://www.baidu.com"]
docker run myip -i
```

##### ONBUILD案例

```dockerfile
FROM centos
RUN yum install -y curl
ENTRYPOINT ["curl","-s","http://ip.cn"]
ONBUILD RUN echo "father image onbuild-----------886"
```

```shell
docker build -f /mydocker/Dockerfile -t myip_father .
```

再进行继承镜像的创建

```dockerfile
FROM myip_father
RUN yum install -y curl
ENTRYPOINT ["curl","-s","http://ip.cn"]
```

```shell
docker build -f /mydocker/Dockerfile -t myip_son .
```

##### 自定义tomcat镜像

**第一步**

```shell
mkdir -p /zzjjuse/mydockerfile/tomcat
```

**第二步**

在上述目录下touch c.txt

**第三步**

将jdk和tomcat安装的压缩包拷贝到该目录中

第四步，创建dockerfile文件

```dockerfile
FROM centos
MAINTAINER zzjj<1782920040@qq.com>
#把宿主机当前上下文的c.txt拷贝到容器/usr/local路径中
COPY c.txt /usr/local/cincontainer.txt
#把Java和tomcat添加到容器中
ADD jdk-8u161-linux-x64.tar.gz /usr/local/
ADD apache-tomcat-8.5.30.tar.gz /usr/local/
#安装vim编辑器
RUN yum -y install vim
#设置工作访问时的WORKDIR路径，登录落脚点
ENV MYPATH /usr/local
WORKDIR $MYPATH
#配置Java和tomcat的环境变量
ENV JAVA_HOME /usr/local/jdk1.8.0_161
ENV CLASSPATH $JAVA_HOME/lib/dt.jar:$JAVA_HOME/lib/tools.jar
ENV CATALINA_HOME /usr/local/apache-tomcat-8.5.30
ENV CATALINA_BASE /usr/local/apache-tomcat-8.5.30
ENV PATH $JAVA_HOME/bin:$CATALINA_HOME/lib:$CATALINA_HOME/bin
#容器运行时监听的端口
EXPOSE 8080
#启动时启动tomcat
#ENTRYPOINT ["/usr/local/apache-tomcat-8.5.30/bin/startup.sh"]
#CMD ["/usr/local/apache-tomcat-8.5.30/bin/catalina.sh","run"]
CMD /usr/local/apache-tomcat-8.5.30/bin/startup.sh && tail -F /usr/local/apache-tomcat-8.5.30/bin/logs/catalina.out
```

**第四步，开始build**

```shell
docker build -t zzjjtomcat .
```

**第五步，开始执行**

```shell
docker run -d -p 9080:8080 --name mytomcat -v /zzjjuse/mydockerfile/tomcat/test:/usr/local/apache-tomcat-8.5.30/webapps/test -v /zzjjuse/mydockerfile/tomcat/tomcatlogs/:/usr/local/apache-tomcat-8.5.30/logs --privileged=true zzjjtomcat
```

**其实执行到这一步就一直有在报错了，暂时我还没有解决，有时间会来再完善的，所以有了后续操作**

**实际场景我们都是从网上下载官方的镜像而不会是自己做一个**

其实这里面有一个注意点，映射路径的时候，一定要在容器webapps新建一个文件夹，不然的话就会将其整个覆盖掉，连tomcat首页也没有了。

##### tomcat启动命令

```shell
docker run -d -p 1112:8080 --name tomcat02 -v /synco/tomcat/tomcat-ormco/ormco02:/usr/local/tomcat/webapps/home -v /synco/tomcat/tomcat-ormco/ormco02/logs/:/usr/local/tomcat/logs --privileged=true tomcat
```

##### 安装mysql

```shell
docker pull registry.cn-hangzhou.aliyuncs.com/hhzz/mysql
```

**mysql启动命令**

参数说明：

-p 12345:3306:：将主机的12345端口映射到docker的3306端口

-v /zzjjuse/mysql/conf:etc/mysql/conf.d：将主机/zzjjuse/mysql目录下的conf/my.cnf挂载到容器的/etc/mysql/conf.d

-v /zzjjuse/mysql/logs:/logs：将主机的/zzjjuse/mysql目录下的logs目录挂载到容器的/logs

-v /zzjjuse/mysql/data:/var/lib/mysql：将主机的/zzjjuse/mysql目录下的data目录挂载到容器的/var/lib/mysql

-e MYSQL_ROOT_PASSWORD=123456：初始化root用户的密码

-d mysql:5.7：后台程序运行mysql

**这种启动方式只适合mysql5.7及一下，更高版本加密规则变了**

```shell
docker run -p 9100:3306 --name mysql-9100 -v /synco/mysql/mysql-9100/conf:/etc/mysql/conf.d -v /synco/mysql/mysql-9100/logs:/logs -v /synco/mysql/mysql-9100/data:/var/lib/mysql -e MYSQL_ROOT_PASSWORD=root -d -d registry.cn-hangzhou.aliyuncs.com/hhzz/mysql
```

**与mysql交互**

```shell
docker exec -it 容器ID /bin/bash	
```

**进行数据备份**

```shell
docker exec mysql容器ID sh -c 'exec mysqldump --all-databases -uroot -p"123456" '> /zzjjuse/all-databases.sql
```

##### 安装redis

```shell
docker pull redis:3.2
```

**启动redis

```shell
docker run -p 6379:6379 -v /synco/redis/redis01/data:/data -d redis redis-server --requirepass "root" --appendonly yes
```

**在主机/zzjjuse/myredis/conf/redis.conf目录下新建redis.conf，对配置进行自定义修改**

```shell
vim /zzjjuse/myredis/conf/redis.conf/redis.conf
```

**连接redis**

```shell
docker exec -it redis容器ID redis-cli
```

##### 安装nginx

```shell
docker pull nginx
docker run --name nginx01 -d -p 80:80 -v /synco/nginx/nginx01/html:/usr/share/nginx/html -v /synco/nginx/nginx01/conf/nginx.conf:/etc/nginx/nginx.conf  -v /synco/nginx/nginx01/logs:/var/log/nginx -v /synco/nginx/nginx01/conf.d:/etc/nginx/conf.d -d nginx
```

#### 安装nacos

```dockerfile
docker pull nacos/nacos-server
docker run --env MODE=standalone --name nacos -d -p 8848:8848 nacos/nacos-server
```

#### 安装sentinel

```shell
docker pull bladex/sentinel-dashboard
docker run --name sentinel -d -p 8858:8858 -d bladex/sentinel-dashboard
```

#### docker安装rabbitmq

```shell
docker run -d --hostname my-rabbit -p 5672:5672 -p 15672:15672 rabbitmq:3.7.3-management
```

#### docker 安装frp

```shell
docker run --restart=always --network host -d -v /etc/frp/frps.ini:/etc/frp/frps.ini --name frps snowdreamtech/frps

# frps.ini
[common]
bind_port = 7000
```

#### 向阿里云推送本地镜像

登录https://cr.console.aliyun.com

```shell
https://cr.console.aliyun.com/repository/cn-qingdao/mrzhou_docker/dev_tomcat/detail
```

#### 杂

##### docker容器中安装vim

```shell
apt-get update
apt-get install vim
```

##### docker容器中jdk的位置

```shell
/usr/lib/jvm
```

#### docker详细部署tomcat

```shell
docker cp tomcat-temp:/usr/local/tomcat/conf /synco/tomcat/tomcat-ormco/ormco02/
docker cp tomcat-temp:/usr/local/tomcat/logs /synco/tomcat/tomcat-ormco/ormco02
docker cp tomcat-temp:/usr/local/tomcat/webapps /synco/tomcat/tomcat-ormco/ormco02

docker run -d -p 9006:8080 --name tomcat-syncophoto -v /synco/tomcat/tomcat-syncophoto/webapps:/usr/local/tomcat/webapps -v /synco/tomcat/tomcat-syncophoto/conf:/usr/local/tomcat/conf -v /synco/tomcat/tomcat-syncophoto/logs:/usr/local/tomcat/logs --privileged=true tomcat
```

```shell
docker cp tomcat-temp:/usr/local/tomcat/conf /synco/tomcat/tomcat-ormco/ormco02/
docker cp tomcat-temp:/usr/local/tomcat/logs /synco/tomcat/tomcat-ormco/ormco02
docker cp tomcat-temp:/usr/local/tomcat/webapps /synco/tomcat/tomcat-ormco/ormco02

docker run -d -p 1113:8080 --name tomcat-fluke01 -v /synco/tomcat/tomcat-fluke/fluke01/webapps:/usr/local/tomcat/webapps -v /synco/tomcat/tomcat-fluke/fluke01/conf:/usr/local/tomcat/conf -v /synco/tomcat/tomcat-fluke/fluke01/logs:/usr/local/tomcat/logs --privileged=true tomcat
```

#### docker查看指定时间后的日志，只显示最后100行：

```shell
docker logs -f -t --since="2018-02-08" --tail=100 CONTAINER_ID
```

#### docker查看最近30分钟的日志:

```shell
docker logs --since 30m CONTAINER_ID
```

#### docker查看某时间之后的日志：

```shell
docker logs -t --since="2018-02-08T13:23:37" CONTAINER_ID
```

#### docker查看某时间段日志：

```shell
docker logs -t --since="2018-02-08T13:23:37" --until "2018-02-09T12:23:37" CONTAINER_ID
```