### vue里的生命周期是什么？

vue实例从创建到销毁的过程称之为vue的生命周期

### vue的生命周期各阶段都做了什么？

`beforeCreate` 实例创建前：这个阶段实例的data、methods是读不到的

`created` 实例创建后：这个阶段已经完成了数据观测(data observer)，属性和方法的运算， watch/event 事件回调。mount挂载阶段还没开始，$el 属性目前不可见，数据并没有在DOM元素上进行渲染

`beforeMount`：在挂载开始之前被调用：相关的 render 函数首次被调用。

`mounted`：el选项的DOM节点 被新创建的 vm.$el 替换，并挂载到实例上去之后调用此生命周期函数。此时实例的数据在DOM节点上进行渲染

`beforeUpdate`：数据更新时调用，但不进行DOM重新渲染，在数据更新时DOM没渲染前可以在这个生命函数里进行状态处理

`updated`：这个状态下数据更新并且DOM重新渲染，当这个生命周期函数被调用时，组件 DOM 已经更新，所以你现在可以执行依赖于 DOM 的操作。当实例每次进行数据更新时updated都会执行

`beforeDestory`：实例销毁之前调用。

`destroyed`：Vue 实例销毁后调用。调用后，Vue 实例指示的所有东西都会解绑定，所有的事件监听器会被移除，所有的子实例也会被销毁。

### vue生命周期在真实场景下的业务应用

`created`：进行ajax请求异步数据的获取、初始化数据

`mounted`：挂载元素内dom节点的获取

`nextTick`：针对单一事件更新数据后立即操作dom

`updated`：任何数据的更新，如果要做统一的业务逻辑处理

`watch`：监听具体数据变化，并做相应的处理

### 总结

| 钩子函数      | 详情                                                         | 场景                                                         | 备注                                                   |
| ------------- | ------------------------------------------------------------ | ------------------------------------------------------------ | ------------------------------------------------------ |
| beforeCreate  | 实例初始化之后，组件被创建时期，这个时候的 el，data，message 都是 underfined | 可以加入 loading 事件；在服务器端的应用场景中，这个时候发送数据请求比较多一些 |                                                        |
| created       | 实例创建完成后，data、methods 被初始化                       | 结束 loading 事件；推荐这个时候发送请求数据，尤其是返回的数据与绑定事件有关时 |                                                        |
| beforeMount   | 挂载初始之前，完成 el 初始化，render 被初次调用              | 也可以发送数据请求                                           | 在服务器端渲染期间不会被调用                           |
| mounted       | 完成挂载                                                     | 获取 el 中 DOM 元素，进行 DOM 操作；如果返回的数据操作依赖 DOM 完成，推荐这个时候发送数据请求 | 在服务器端渲染期间不会被调用                           |
| beofreUpdate  | 挂载开始之前调用                                             | 挂载完成之前访问现有 DOM ，比如手动移除已添加的事件监听器；也可以进一步修改数据 | 在服务器渲染期间不会被调用，只有初次渲染会在服务端调用 |
| updated       | 由于数据更改，重新渲染界面时调用                             | 可执行依赖于 DOM 的操作                                      | 服务器端渲染期间不会被调用                             |
| activated     | keep-alive 组件激活时调用                                    |                                                              | 服务器端渲染期间不会被调用                             |
| deactivated   | 组件停用时调用                                               |                                                              | 服务器端渲染期间不会被调用                             |
| beforeDestroy | 实例销毁之前调用                                             | 删除钱发出确认信息；清理定时器                               | 服务器端渲染期间不会被调用                             |
| destroyed     | 实例销毁后调有。调用后，所有东西都会被解绑，所有的事件监听器会被移动，子实例也会被销毁 | 提示已删除                                                   | 服务器端渲染期间不会被调用                             |
| errorCaptured | 当捕获一个来自子孙组件的错误时被调用。收到三个参数：错误对象、发生错误的组件实例以及一个包含错误来源信息的字符串。可返回 false 以阻止该错误继续向上传播。 | 修改组件状态                                                 | 2.5.0新增                                              |